import 'package:arcana_auth_flutter/common.dart';
import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';

class ArcanaSdk {
  WebViewController? controller;
  late OverlayEntry _overlayEntry;
  late OverlayState _overlayState;
  bool _isVisible = false;
  bool fullScreen = false;
  double height = 200;
  double width = 200;
  Alignment alignment = Alignment.center;
  late BuildContext context;
  late Function onLoginSuccess;
  late Function onLog;
  Function? onUserInfoResponse;
  Function? onTransactionResponse;
  Function? getAccountResponse;
  Function? onLogoutResponse;
  Function? onSendRequestResponse;

  initWeb(
      {required bool fullScreen,
      required double height,
      required double width,
      required Alignment alignment,
      required BuildContext context,
      required Function() onLoginSuccess,
      required Function() onLoginFailed,
      required Function(String error) onError,
      required ArcanaConfig config,
      required bool isPasswordLessLogin,
      required String clientId,
      String? email,
      required Function(String value) onLog}) {
    this.context = context;
    this.fullScreen = fullScreen;
    this.height = height;
    this.width = width;
    this.alignment = alignment;
    this.onLoginSuccess = onLoginSuccess;
    this.onLog = onLog;
    controller = WebViewController()
      ..setJavaScriptMode(JavaScriptMode.unrestricted)
      ..setBackgroundColor(const Color(0x00000000))
      ..setUserAgent(
          'Mozilla/5.0 (Linux; Android 5.1.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/91.0.4472.164 Mobile Safari/535.19')
      ..addJavaScriptChannel('xarFlutter', onMessageReceived: (message) {
        print('xarFlutter: ' + message.message);
        this.onLog(message.message);
        if(message.message.contains('show')){
          showArcanaView();
        }else if(message.message.contains('hide')){
          hideArcanaView();
        }else if(message.message.contains('login_complete')){
          onLoginSuccess();
        }else if(message.message.contains('get_user_info')){
          if(onUserInfoResponse!=null){
            onUserInfoResponse!(message.message.replaceAll("get_user_info-", ""));
          }
        }else if(message.message.contains('sendTransaction')){
          if(onTransactionResponse!=null){
            onTransactionResponse!(message.message.replaceAll("sendTransaction-", ""));
          }
        }else if(message.message.contains('sendRequest')){
          if(onSendRequestResponse!=null){
            onSendRequestResponse!(message.message.replaceAll("sendRequest-", ""));
          }
        }else if(message.message.contains('getAccount')){
          if(getAccountResponse!=null){
            getAccountResponse!(message.message.replaceAll("getAccount-", ""));
          }
        }else if(message.message.contains('logout')){
          if(onLogoutResponse!=null){
            // destroyView();
            onLogoutResponse!("Logout Successfully");
          }
        }
      })
      ..setNavigationDelegate(
        NavigationDelegate(
          onProgress: (int progress) {},
          onPageStarted: (String url) {},
          onPageFinished: (String url) {},
          onWebResourceError: (WebResourceError error) {},
        ),
      )
      ..loadRequest(Uri.parse(getUrl(isPasswordLessLogin,clientId,email,config,onError)));

  }

  clearCache() {
    controller?.clearCache();
    var webViewCookie = WebViewCookieManager();
    webViewCookie.clearCookies();
    controller?.reload();
    destroyView();
  }

  OverlayEntry _createOverlayEntry() {
    return OverlayEntry(
      builder: (context) => Align(
          alignment: alignment,
          child: Container(
            height: fullScreen ? null : height,
            width: fullScreen ? null : width,
            child: Scaffold(
              appBar: fullScreen
                  ? AppBar(
                      leading: CloseButton(
                          onPressed: () {
                            hideArcanaView();
                          },
                          color: Colors.black),
                    )
                  : null,
              body: Container(
                child: controller!=null?WebViewWidget(controller: controller??WebViewController()):Container(),
              ),
            ),
          )),
    );
  }

  showArcanaView() async {
    if(controller!=null){
      if (!_isVisible) {
        _overlayState = Overlay.of(context);
        _overlayEntry = _createOverlayEntry();
        _overlayState.insert(_overlayEntry);
        _isVisible = true;
      }
    }
  }

  void resize(bool fullScreen,double height,double width){
    this.fullScreen=fullScreen;
    this.height=height;
    this.width=width;
    hideArcanaView();
    showArcanaView();
  }

  void hideArcanaView() {
    _isVisible = false;
    _overlayEntry.remove();
  }

  getWebViewVisibleStatus(){
    return _isVisible;
  }

  void destroyView() {
    _isVisible = false;
    _overlayEntry.remove();
  }

  void injectJavaScript(String data) {
    controller?.runJavaScript(
        '(function() {window.dispatchEvent(new MessageEvent("message", {data: ${data}}));})()');
  }

  void userDefinedRequest(String data) {
    injectJavaScript(data);
  }

  void sendRequest(id, {required Function(String response) onSendRequestResponse}) {
    this.onSendRequestResponse=onSendRequestResponse;
    injectJavaScript('{ id: "sendRequest-${id}", method: "" , params: [] }');
  }

  void getUserInfo(id,{required Function(String response) onUserInfoResponse}) {
    this.onUserInfoResponse=onUserInfoResponse;
    injectJavaScript(
        '{type: "request", data: { id: "get_user_info-${id}",type:"newData", method: "get_user_info", params: [] }}');
  }

  void sendTransaction(id, from, to, value,{required Function(String response) onTransactionResponse}) {
    this.onTransactionResponse=onTransactionResponse;
    injectJavaScript(
        '{type: "request",data: {id:"sendTransaction-${id}",method: "eth_sendTransaction",params: [{from:${from},to:${to},value:${value}},],},}');
  }

  void getAccount(id,{required Function(String response) getAccountResponse}) {
    this.getAccountResponse=getAccountResponse;
    injectJavaScript(
        '{type: "request",data: { id:"getAccount-${id}", method: "eth_accounts", params: [] },}');
  }

  void logOut({required Function(String response) onLogoutResponse}) {
    this.onLogoutResponse=onLogoutResponse;
    injectJavaScript('{type: "logout",id:"logout"}');
  }

  String getUrl(bool isPasswordLessLogin, String clientId, String? email, ArcanaConfig config, Function onError) {
    if(isPasswordLessLogin){
      if(email!=null && email.length!=0){
        return "${getConfig(config)}/start?clientId=${clientId}&loginSrc=flutter&type=passwordless&email=${email}&theme=dark";
      }else{
        onError("Invalid Email");
        return "";
      }
    }else{
      return "${getConfig(config)}/start?clientId=${clientId}&loginSrc=flutter&type=google&theme=dark";

    }
  }
}
